@@ -1,122 +1,176 @@
 package Mojolicious::Plugin::RoutesAuthDBI;
 use Mojo::Base 'Mojolicious::Plugin::Authentication';
 use Mojo::Loader qw(load_class);
+use Mojo::Util qw(hmac_sha1_sum);
+use Hash::Merge qw( merge );
 
-our $VERSION = '0.447';
+our $VERSION = '0.501';
 
-my $access;# 
 my $pkg = __PACKAGE__;
-my $conf ;# set on ->registrer
 
-my $fail_auth = {format=>'txt', text=>"Deny at auth step. Please sign in!!!\n"};
-my $fail_auth_cb = sub {shift->render(%$fail_auth);};
-my $fail_access_cb = sub {
-  my ($c, $route, $args, $u) = @_;
-  $c->app->log->debug(sprintf "Deny [%s] for user id=[%s]; args=[%s]; defaults=[%s]",
-    $route->pattern->unparsed,
-    $u->{id},
-    $c->dumper($args) =~ s/\s+//gr,
-    $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
-  );
-  $c->render(format=>'txt', text=>"You don`t have access on this route (url, action) !!!\n");
-  
+has [qw(app dbh conf)];
+
+has default => sub {
+  my $self = shift;
+  {
+  auth => {
+    stash_key => $pkg,
+    current_user_fn => 'auth_user',
+    load_user => \&load_user,
+    validate_user => \&validate_user,
+  },
+  access => {
+    namespace => $pkg,
+    module => 'Access',
+    fail_auth_cb => sub {
+      shift->render(format=>'txt', text=>"Access deny at auth step. Please sign in!!!\n");
+    },
+    fail_access_cb => sub {
+      shift->render(format=>'txt', text=>"You don`t have access on this route (url, action) !!!\n");
+    },
+    import => [qw(load_user validate_user)],
+  },
+  admin => {
+    namespace => $pkg,
+    controller => 'Admin',
+    prefix => lc($self->conf->{admin}{controller} || 'admin'),
+    trust => hmac_sha1_sum('admin', $self->app->secrets->[0]),
+  },
+  pos => {namespace => $pkg, module => 'POS::Pg', },
+  sth => {namespace => $pkg, module => 'Sth', },
+  template => {schema => 'public'},
+}};
+
+has merge_conf => sub {#hashref
+  my $self = shift;
+  merge($self->conf, $self->default);
 };
 
+has pos => sub {# object DBIx::POS::Template
+  my $self = shift;
+  my $pos = $self->merge_conf->{'pos'};
+  my $class = $self->_class($pos);
+  $class->new;
+};
+
+has sth => sub {# object Sth
+  my $self = shift;
+  my $sth = $self->merge_conf->{'sth'};
+  my $class = $self->_class($sth);
+  my $template = $self->merge_conf->{template};
+  $class->new($self->dbh, $self->pos, %$template);
+};
+
+has access => sub {# object
+  my $self = shift;
+  my $access = $self->merge_conf->{'access'};
+  my $class = $self->_class($access);
+  $class->import( @{$access->{import}});
+  bless $access, $class;
+  $access->{dbh} = $self->dbh;
+  $access->{sth} = $self->sth;
+  $access->{app} = $self->app;
+  return $access->init;
+};
+
+has admin => sub {
+  my $self = shift;
+  my $admin = $self->merge_conf->{'admin'};
+  $admin->{module} ||= $admin->{controller};
+  my $class = $self->_class($admin);
+  bless $admin, $class;
+  $admin->{dbh} = $self->dbh;
+  $admin->{sth} = $self->sth;
+  
+  return $admin->init;
+};
+
+
 sub register {
-  my ($self, $app,) = (shift, shift);
-  $conf = shift; # global
-  $conf->{dbh} ||= $app->dbh;
-  $conf->{dbh} = $conf->{dbh}($app)
-    if ref($conf->{dbh}) eq 'CODE';
-  $conf->{pos} ||= {};
-  $conf->{pos}{schema} ||= 'public';
-  $conf->{pos}{file} ||= 'POS/Pg.pm';
-  die "Plugin must work with arg dbh, see SYNOPSIS" unless $conf->{dbh};
-  $conf->{access} ||= {};
-  $conf->{access}{namespace} ||= $pkg unless $conf->{access}{module};
-  $conf->{access}{module} ||= 'Access';
-  $conf->{access}{dbh} = $conf->{dbh};
-  $conf->{access}{fail_auth_cb} ||= $fail_auth_cb;
-  $conf->{access}{fail_access_cb} ||= $fail_access_cb;
-  $conf->{access}{pos} ||= $conf->{pos};
-  # class obiect
-  $access ||= $self->access_instance($app, $conf->{access});
+  my $self = shift;
+  $self->app(shift);
+  $self->conf(shift); # global
   
-  $conf->{auth}{stash_key} ||= $pkg;
-  $conf->{auth}{current_user_fn} ||= 'auth_user';
-  $conf->{auth}{load_user} ||= \&load_user;
-  $conf->{auth}{validate_user} ||= \&validate_user;
-  $conf->{auth}{fail_render} ||= $fail_auth;
-  $self->SUPER::register($app, $conf->{auth});
+  $self->dbh($self->conf->{dbh} || $self->app->dbh);
+  $self->dbh($self->dbh->($self->app))
+    if ref($self->dbh) eq 'CODE';
+  die "Plugin must work with dbh, see SYNOPSIS" unless $self->dbh;
   
-  $app->routes->add_condition(access => \&access);
-  $access->{'app.routes'} = $app->routes;
-  $access->apply_ns($app);
-  $access->apply_route($app, $_) for @{ $access->db_routes };
+  my $access = $self->access;
   
-  if ($conf->{admin}) {
-    $conf->{admin}{namespace} ||= $pkg;
-    $conf->{admin}{controller} ||= 'Admin';
-    $conf->{admin}{dbh} = $conf->{dbh};
-    $conf->{admin}{sth} = $access->{sth};
-    $conf->{admin}{prefix} ||= lc($conf->{admin}{controller});
-    $conf->{admin}{trust} ||= $app->secrets->[0];
-    $conf->{admin}{pos} ||= $conf->{pos};
-    my $admin ||= $self->admin_controller($app, $conf->{admin});
-    $access->apply_route($app, $_) for $admin->self_routes;
+  #~ die $self->app->dumper($access);
+  
+  $self->SUPER::register($self->app, $self->merge_conf->{auth});
+  
+  $self->app->routes->add_condition(access => sub {$self->cond_access(@_)});
+  $access->apply_ns();
+  $access->apply_route($_) for @{ $access->db_routes };
+  
+  if ($self->conf->{admin}) {
+    my $admin = $self->admin;
+    $access->apply_route($_) for $admin->self_routes;
   }
   
-  $app->helper('access', sub {$access});
+  $self->app->helper('access', sub {$access});
   
   return $self, $access;
 
 }
 
-sub access_instance {# auth, routes and access methods
-  my ($self, $app, $conf) = @_;
-  my $class  = _load_mod( $conf->{namespace}, $conf->{module});
-  $class->import( qw(load_user validate_user) );
-  return (bless $conf, $class)->init_class;
-}
-
-sub admin_controller {# web interface :)
-  my ($self, $app, $conf) = @_;
-  my $class  = _load_mod( $conf->{namespace}, $conf->{controller});
-  return (bless $conf, $class)->init_class;
-}
-
-sub _load_mod {
-  my ($ns, $mod) = @_;
-  my $class  = join '::', $ns, $mod;
-  require join '/', $ns =~ s/::/\//gr, $mod.'.pm';
+sub _class {
+  my $self = shift;
+  my $conf = shift;
+  my $class  = join '::', $conf->{namespace}, $conf->{module};
+  my $e; $e = load_class($class)# success undef
+    and die $e;
   return $class;
 }
 
-
 # 
-sub access {# add_condition
+sub cond_access {# add_condition
+  my $self= shift;
   my ($route, $c, $captures, $args) = @_;
-  #~ $c->app->log->debug($c->dumper($route));#$route->pattern->defaults
-  # 1. по паролю выставить куки
-  # 2. по кукам выставить пользователя
-  my $meth = $conf->{auth}{current_user_fn};
-  my $u = $c->$meth;
-  if (ref $args eq 'CODE') {
-    $args->($u, @_)
-      or $conf->{access}{fail_auth_cb}->($c, )
-      and return undef;
-    return 0x01;
-  }
-  # 3. если не проверять доступ вернуть 1
-  return 1 unless $args->{auth};
+  my $conf = $self->merge_conf;
+  my $app = $c->app;
+  my $access = $self->access;
+  #~ $app->log->debug($c->dumper($route));#$route->pattern->defaults
+  
+  my $auth_helper = $conf->{auth}{current_user_fn};
+  my $u = $c->$auth_helper;
+  $app->log->debug(sprintf(qq[Access allow [%s] for {auth}=false],
+    $route->pattern->unparsed,
+  ))
+    and return 1 # не проверяем доступ
+    unless $args->{auth};
+  
   # не авторизовался
-  $conf->{access}{fail_auth_cb}->($c, )
+  #~ $app->log->debug("Access deny for non auth user")
+  $self->deny_log($route, $args, $u)
+    and $access->{fail_auth_cb}->($c, )
     and return undef
     unless $u;
-  # допустить если {auth=>'only'}
-  return 1 if lc($args->{auth}) eq 'only';
+  
   #  получить все группы пользователя
   $access->load_user_roles($u);
+  
+  # допустить если {auth=>'only'}
+  $app->log->debug(sprintf(qq[Access allow [%s] for {auth}='only'],
+    $route->pattern->unparsed,
+  ))
+    and return 1
+    if lc($args->{auth}) eq 'only';
+  
+  if (ref $args eq 'CODE') {
+    $args->($u, @_)
+      or $self->deny_log($route, $args, $u)
+      and $access->{fail_auth_cb}->($c, )
+      #~ and $app->log->debug("Access deny user by callback condition")
+      and return undef;
+    $app->log->debug(sprintf(qq[Access allow [%s] by callback condition],
+      $route->pattern->unparsed,
+    ));
+    return 0x01;
+  }
 
   my $id2 = [$u->{id}, map($_->{id}, grep !$_->{disable},@{$u->{roles}})];
   my $id1 = [grep $_, @$args{qw(id route_id action_id controller_id namespace_id)}];
@@ -124,7 +178,7 @@ sub access {# add_condition
   # explicit acces to route
   scalar @$id1
     && $access->access_explicit($id1, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] joined id1=%s; args=[%s]; defaults=%s",
+    && $app->log->debug(sprintf "Access allow [%s] for roles=[%s] joined id1=%s; args=[%s]; defaults=%s",
       $route->pattern->unparsed,
       $c->dumper($id2) =~ s/\s+//gr,
       $c->dumper($id1) =~ s/\s+//gr,
@@ -136,7 +190,7 @@ sub access {# add_condition
   # Access to non db route by role
   $args->{role}
     && $access->access_role($args->{role}, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] by role [%s] joined roles=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] by role [%s] joined roles=[%s]",
       $route->pattern->unparsed,
       $args->{role},
     )
@@ -146,14 +200,15 @@ sub access {# add_condition
   my $controller = $args->{controller} || ucfirst(lc($route->pattern->defaults->{controller}));
   my $namespace = $args->{namespace} || $route->pattern->defaults->{namespace};
   if ($controller && !$namespace) {
-    (load_class($_.'::'.$controller) or ($namespace = $_) and last) for @{ $access->{'app.routes'}->namespaces };
+    (load_class($_.'::'.$controller) or ($namespace = $_) and last) for @{ $app->routes->namespaces };
   }
-  $conf->{access}{fail_access_cb}->($c, $route, $args, $u)
+  $self->deny_log($route, $args, $u)
+    and $access->{fail_access_cb}->($c,)
     and return undef
     unless $controller && $namespace;# failed load class
 
   $access->access_namespace($namespace, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by namespace=[%s]; args=[%s]; defaults=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] for roles=[%s] by namespace=[%s]; args=[%s]; defaults=[%s]",
       $route->pattern->unparsed,
       $c->dumper($id2) =~ s/\s+//gr,
       $namespace,
@@ -163,7 +218,7 @@ sub access {# add_condition
     && return 1;
   
   $access->access_controller($namespace, $controller, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by namespace=[%s] and controller=[%s]; args=[%s]; defaults=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] for roles=[%s] by namespace=[%s] and controller=[%s]; args=[%s]; defaults=[%s]",
       $route->pattern->unparsed,
       $c->dumper($id2) =~ s/\s+//gr,
       $namespace, $controller,
@@ -175,7 +230,7 @@ sub access {# add_condition
   # еще раз контроллер, который тут без namespace и в базе без namespace ------> доступ из любого места
   $args->{namespace} || $route->pattern->defaults->{namespace}
     || $access->access_controller(undef, $controller, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by controller=[%s] without namespace on db; args=[%s]; defaults=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] for roles=[%s] by controller=[%s] without namespace on db; args=[%s]; defaults=[%s]",
       $route->pattern->unparsed,
       $c->dumper($id2) =~ s/\s+//gr,
       $controller,
@@ -185,11 +240,12 @@ sub access {# add_condition
     && return 1;
   
   my $action = $args->{action} || $route->pattern->defaults->{action}
-    or $conf->{access}{fail_access_cb}->($c, $route, $args, $u)
+    or $self->deny_log($route, $args, $u)
+    and $access->{fail_access_cb}->($c,)
     and return undef;
   
   $access->access_action($namespace, $controller, $action, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by namespace=[%s] and controller=[%s] and action=[%s]; args=[%s]; defaults=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] for roles=[%s] by namespace=[%s] and controller=[%s] and action=[%s]; args=[%s]; defaults=[%s]",
       $route->pattern->unparsed,
       $c->dumper($id2) =~ s/\s+//gr,
       $namespace , $controller, $action,
@@ -201,7 +257,7 @@ sub access {# add_condition
   # еще раз контроллер, который тут без namespace и в базе без namespace ------> доступ из любого места
   $args->{namespace} || $route->pattern->defaults->{namespace}
     && $access->access_action(undef, $controller, $action, $id2)
-    && $c->app->log->debug(sprintf "Allow [%s] for roles=[%s] by (namespace=[any]) controller=[%s] and action=[%s]; args=[%s]; defaults=[%s]",
+    && $app->log->debug(sprintf "Access allow [%s] for roles=[%s] by (namespace=[any]) controller=[%s] and action=[%s]; args=[%s]; defaults=[%s]",
       $route->pattern->unparsed,
       $c->dumper($id2) =~ s/\s+//gr,
       $controller, $action,
@@ -209,11 +265,23 @@ sub access {# add_condition
       $c->dumper($route->pattern->defaults) =~ s/\s+//gr,
     )
     && return 1;
-  
-  $conf->{access}{fail_access_cb}->($c, $route, $args, $u);
+    
+  $self->deny_log($route, $args, $u);
+  $access->{fail_access_cb}->($c,);
   return undef;
 }
 
+sub deny_log {
+  my $self = shift;
+  my ($route, $args, $u) = @_;
+  my $app = $self->app;
+  $app->log->debug(sprintf "Access deny [%s] for user id=[%s]; args=[%s]; defaults=[%s]",
+    $route->pattern->unparsed,
+    $u ? $u->{id} : 'non auth',
+    $app->dumper($args) =~ s/\s+//gr,
+    $app->dumper($route->pattern->defaults) =~ s/\s+//gr,
+  );
+}
 
 1;
 
@@ -229,7 +297,7 @@ sub access {# add_condition
 
 =head1 VERSION
 
-0.447
+0.501
 
 =head1 NAME
 
@@ -247,11 +315,14 @@ First of all you will see L<SVG|https://github.com/mche/Mojolicious-Plugin-Route
     access => {...},
     admin => {...},
     pos => {...},
+    template => {...},
   );
 
 
 =head2 PLUGIN OPTIONS
 
+One option C<dbh> is mandatory, all other - optional.
+
 =head3 dbh
 
 Handler DBI connection where are tables: controllers, actions, routes, users, roles, refs.
@@ -318,7 +389,9 @@ See L<Mojolicious::Plugin::RoutesAuthDBI::Admin> for detail options list.
 
 Hashref options for POS-dictionary instance. See L<Mojolicious::Plugin::RoutesAuthDBI::Sth>.
 
-=back
+=head3 template
+
+Hashref options for template of POS-dictionary. See L<Mojolicious::Plugin::RoutesAuthDBI::POS::Pg>.
 
 =head1 INSTALL
 
