@@ -1,127 +1,30 @@
 package Mojolicious::Plugin::RoutesAuthDBI::Admin;
 use Mojo::Base 'Mojolicious::Controller';
-use Mojolicious::Plugin::RoutesAuthDBI::Sth;#  sth cache
+#~ use Mojolicious::Plugin::RoutesAuthDBI::Sth;#  sth cache
 
-my $dbh; # one per class
 my $pkg = __PACKAGE__;
-my $init_conf;
-my $sth;#sth hub
+my ($dbh, $sth, $init_conf);
+has [qw(dbh sth)];
 
-=pod
+sub init {# from plugin! init Class vars
+  my $self = shift;
+  my %args = @_;
 
-=encoding utf8
-
-=head1 Mojolicious::Plugin::RoutesAuthDBI::Admin
-
-¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
-
-=head1 WARN
-
-More or less complete!
-
-=head1 NAME
-
-Mojolicious::Plugin::RoutesAuthDBI::Admin - is a Mojolicious::Controller for manage admin operations on DBI tables: namespaces, controllers, actions, routes, roles, users.
-
-=head1 DB DESIGN DIAGRAM
-
-See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagram.svg>
-
-=head1 SYNOPSIS
-
-    $app->plugin('RoutesAuthDBI', 
-        ...
-        admin => {< options below >},
-        ...
-    );
-
-
-=over 4
-
-=item * B<namespace> - default 'Mojolicious::Plugin::RoutesAuthDBI',
-
-=item * B<controller> - module controller name, default 'Admin',
-
-Both above options determining the module controller for web actions on tables routes, roles, users and refs between them.
-
-=item * B<prefix> -is a prefix for admin urls of this module. Default as name of controller lowcase.
-
-=item * B<trust> is a url subprefix for trust admin urls of this module. See defaults below.
-
-=back
-
-=head2 Defaults
-
-    admin = > {
-        namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
-        module => 'Admin',
-        prefix => 'admin', # lc(<module>)
-        trust => $app->secrets->[0],
-        pos => {...}, # sets from plugin options
-    },
-    
-    admin = {}, # empty hashref sets defaults above
-    
-    admin => undef, # disables routing of admin controller
-    
-    admin = > {prefix=>'myadmin', trust => 'foooobaaar'},# admin urls like: /myadmin/foooobaaar/.....
-
-
-
-=head1 METHODS NEEDS IN PLUGIN
-
-=over 4
-
-=item * B<self_routes()> - builtin to this access controller routes. Return array of hashrefs routes records for apply route on app. Depends on conf options I<prefix> and I<trust>.
-
-=back
-
-=head1 SEE ALSO
-
-L<Mojolicious::Plugin::RoutesAuthDBI>
-
-L<Mojolicious::Plugin::RoutesAuthDBI::Sth>
-
-=head1 AUTHOR
-
-Михаил Че (Mikhail Che), C<< <mche [on] cpan.org> >>
-
-=head1 BUGS / CONTRIBUTING
-
-Please report any bugs or feature requests at L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/issues>. Pull requests also welcome.
-
-=head1 COPYRIGHT
-
-Copyright 2016 Mikhail Che.
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
-=cut
-
-sub init_class {# from plugin! init Class vars
-	my $c = shift;
-	my %args = @_;
-  $init_conf ||= $c;
-  $c->{prefix} =~ s/^\///;
-  $c->{trust} =~ s/\W/-/g;
-  $c->{pos} ||= {};
-  $c->{pos}{schema} ||= 'public';
-  #~ $c->{pos}{schema} = qq{"$c->{pos}{schema}".};
-  $c->{pos}{file} ||= 'POS/Pg.pm';
-	$c->{dbh} ||= $dbh ||=  $args{dbh};
-	$dbh ||= $c->{dbh};
-	#~ $c->{sth} ||= $sth ||= $args{sth} ||= (bless [$dbh, {}, $c->{schema},], $c->{namespace}.'::Sth')->init(pos => $c->{pos} || 'POS/Pg.pm');#sth cache
-  $c->{sth} ||= $sth ||= $args{sth} || Mojolicious::Plugin::RoutesAuthDBI::Sth->new($dbh, %{$c->{pos}});
-	$sth ||= $c->{sth};
-    
-	return $c;
+  $self->dbh($self->{dbh} || $args{dbh});
+  $dbh = $self->dbh
+    or die "Нет DBI handler";
+  $self->sth($self->{sth} || $args{sth});
+  $sth = $self->sth
+    or die "Нет STH";
+  #~ $app = $self->app($self->{app} || $args{app});
+  $init_conf = $self;
+  return $self;
 }
 
 
 sub index {
   my $c = shift;
-  
+  no warnings;
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -138,7 +41,7 @@ TXT
     and return
     if $c->is_user_authenticated;
   
-  $c->render(format=>'txt', text=>__PACKAGE__."\n\nYou are not signed!!! To sign in/up go to /sign/<login>/<pass>");
+  $c->render(format=>'txt', text=>__PACKAGE__."\n\nYou are not signed!!!\n\nTo sign in go to /sign/in/<login>/<pass>\n");
 }
 
 sub sign {
@@ -831,7 +734,7 @@ sub self_routes {# from plugin!
   my $trust = $init_conf->{trust};
 
   my $t = <<TABLE;
-/$prefix	index	admin home	1	View main page
+/$prefix	index	admin home	1	Main page
 #
 # Namespaces, controllers, actions
 #
@@ -895,6 +798,97 @@ TABLE
   return @r;
 }
 
-
-
 1;
+
+=pod
+
+=encoding utf8
+
+=head1 Mojolicious::Plugin::RoutesAuthDBI::Admin
+
+¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
+
+=head1 WARN
+
+More or less complete!
+
+=head1 NAME
+
+Mojolicious::Plugin::RoutesAuthDBI::Admin - is a Mojolicious::Controller for manage admin operations on DBI tables: namespaces, controllers, actions, routes, roles, users.
+
+=head1 DB DESIGN DIAGRAM
+
+See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagram.svg>
+
+=head1 SYNOPSIS
+
+    $app->plugin('RoutesAuthDBI', 
+        ...
+        admin => {< options below >},
+        ...
+    );
+
+
+=over 4
+
+=item * B<namespace> - default 'Mojolicious::Plugin::RoutesAuthDBI',
+
+=item * B<controller> - module controller name, default 'Admin',
+
+Both above options determining the module controller for web actions on tables routes, roles, users and refs between them.
+
+=item * B<prefix> -is a prefix for admin urls of this module. Default as name of controller lowcase.
+
+=item * B<trust> is a url subprefix for trust admin urls of this module. See defaults below.
+
+=back
+
+=head2 Defaults
+
+    admin = > {
+        namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
+        module => 'Admin',
+        prefix => 'admin', # lc(<module>)
+        trust => $app->secrets->[0],
+        pos => {...}, # sets from plugin options
+    },
+    
+    admin = {}, # empty hashref sets defaults above
+    
+    admin => undef, # disables routing of admin controller
+    
+    admin = > {prefix=>'myadmin', trust => 'foooobaaar'},# admin urls like: /myadmin/foooobaaar/.....
+
+
+
+=head1 METHODS NEEDS IN PLUGIN
+
+=over 4
+
+=item * B<self_routes()> - builtin to this access controller routes. Return array of hashrefs routes records for apply route on app. Depends on conf options I<prefix> and I<trust>.
+
+=back
+
+=head1 SEE ALSO
+
+L<Mojolicious::Plugin::RoutesAuthDBI>
+
+L<Mojolicious::Plugin::RoutesAuthDBI::Sth>
+
+=head1 AUTHOR
+
+Михаил Че (Mikhail Che), C<< <mche [on] cpan.org> >>
+
+=head1 BUGS / CONTRIBUTING
+
+Please report any bugs or feature requests at L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/issues>. Pull requests also welcome.
+
+=head1 COPYRIGHT
+
+Copyright 2016 Mikhail Che.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+=cut
+
